if (not RaidInfo.tFubar) then return end

local L = AceLibrary("AceLocale-2.2"):new("RaidInfo")
local Tablet = AceLibrary("Tablet-2.0")
local TabletCat = nil
local PC = AceLibrary("PaintChips-2.0")

function RaidInfo:FuOnInitialize(...)
	self.FuBar.OnMenuRequest = self:BuildOptions()
end

function RaidInfo:FuOnDataUpdate(...)
	--
end

function RaidInfo:FuOnTextUpdate(...)
	local fuText = self:FuName()
	
	self.FuBar:SetText(fuText)
end

function RaidInfo:FuOnTooltipUpdate(...)
	Tablet:SetTitle(self:FuName())
	Tablet:SetTitleColor(163/255, 53/255, 238/255) -- Epic for the right feeling
	
	self:TabletSetColorJustifyCheck("ItemQuality6", "LEFT", true)
	
	local cat = Tablet:AddCategory('columns', 1)
	
	if (self:CurrentGroupType()) then
		self:NewTabletCategory('columns', 3)
	  self:TabletSetColorJustifyCheck("ItemQuality6", nil, nil)
		self:NewTabletLine(
			  L["TabletCatName"]
			, L["TabletCatSpell"]
			, L["TabletCatCD"]
		)
		
	  for raidIndex=1, GetNumRaidMembers() do
		  local name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(raidIndex)
		  self:TabletSetColorJustifyCheck(fileName, nil, nil)
		  
		  if (self.knowledgeBase[name]) then
		  	for spellName, spellInfo in pairs(self.knowledgeBase[name]) do
		  		local timeToRun = spellInfo["Done"] - GetTime()
		  		local barName = self:BarName(name, spellName)
		  		local _, _, _, running = self:CandyBarStatus(barName)
		  		
				  self:NewTabletLine(
						  name
						, spellName
						, self:GetFormattedTime(timeToRun)
						, function() self:FuRowClick(name, spellName) end
						, running
					)
				end
			end
		end
		
		self:TabletExtraHint(L["FuHintClick"])
  else
  	self:NewTabletCategory('columns', 1)
		  
	  self:TabletSetColorJustifyCheck("ItemQuality6", nil, nil)
	  self:NewTabletLine(L["Nothing to display"])
  end
end

function RaidInfo:FuOnClick(...)
	
end

function RaidInfo:FuOnDoubleClick(...) -- "button"
	--
end

function RaidInfo:FuOnMouseDown(...) -- "button"
	--
end

function RaidInfo:FuOnReceiveDrag(...)
	--
end

function RaidInfo:FuRowClick(clickName, spellName)
	self:ToggleVisibility(clickName, spellName)
	self.FuBar:Update()
end

function RaidInfo:TabletSetColorJustifyCheck(pcColor, justify, hasCheck)
	if (not self.tabletOptions) then self.tabletOptions = { } end
	if (type(pcColor) == "string") then self.tabletOptions.pcColor = pcColor end
	if (type(justify) == "string") then self.tabletOptions.justify = justify end
	if (type(hasCheck) == "boolean") then self.tabletOptions.hasCheck = hasCheck end
end

function RaidInfo:NewTabletCategory(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
	local args = {}
	tinsert(args, arg1)
	tinsert(args, arg2)
	tinsert(args, arg3)
	tinsert(args, arg4)
	tinsert(args, arg5)
	tinsert(args, arg6)
	tinsert(args, arg7)
	tinsert(args, arg8)
	tinsert(args, arg9)
	
	local info = { }
	for _, locArg in pairs(args) do
		if (type(locArg) == "number") then
			info["columns"] = locArg
		elseif (type(locArg) == "boolean") then
			info["hideBlankLine"] = locArg
		end
	end
	
	return info
end

function RaidInfo:NewTabletCategory(...)
	TabletCat = Tablet:AddCategory(...)
end

function RaidInfo:NewTabletLine(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
	if (not self.tabletOptions) then self.tabletOptions = { } end
	
	local args = {}
	tinsert(args, arg1)
	tinsert(args, arg2)
	tinsert(args, arg3)
	tinsert(args, arg4)
	tinsert(args, arg5)
	tinsert(args, arg6)
	tinsert(args, arg7)
	tinsert(args, arg8)
	tinsert(args, arg9)
	
	local info = { }
	local txtCounter = 1
	local _, textR, textG, textB = PC:GetRGBPercent(self.tabletOptions.pcColor)
	
	for _, locArg in pairs(args) do
		if (type(locArg) == "function") then
			info["func"] = locArg
		elseif (type(locArg) == "boolean") then
			info["isRadio"] = locArg
			info["checked"] = locArg
		else
			local textString = "text"..txtCounter
			local justifyString = "justify"..txtCounter
			local textRString = "text"..txtCounter.."R"
			local textGString = "text"..txtCounter.."G"
			local textBString = "text"..txtCounter.."B"
			
			if (txtCounter == 1) then
				textString = "text"
				justifyString = "justify"
				textRString = "textR"
				textGString = "textG"
				textBString = "textB"
			end
			
			info[textString] = locArg
			info[justifyString] = self.tabletOptions.justify
			info[textRString] = textR
			info[textGString] = textG
			info[textBString] = textB
			
			txtCounter = txtCounter + 1
		end
	end
	info["hasCheck"] = self.tabletOptions.hasCheck

	TabletCat:AddLine(info)
end

function RaidInfo:TabletExtraHint(hint, func)
	self:TabletSetColorJustifyCheck("green", "LEFT", nil)
	self:NewTabletCategory('columns', 1, 'hideBlankLine', true)
	self:NewTabletLine("Hint: "..hint, func)
end

function RaidInfo:TabletExtraHeadline(headline, func)
	self:TabletSetColorJustifyCheck("ItemQuality6", "CENTER", nil)
	self:NewTabletCategory('columns', 1)
	self:NewTabletLine(headline, func)
end